searchData={"items":[{"type":"behaviour","doc":"Smokestack provides a way to define test factories for your [Ash\nResources](https://ash-hq.org/docs/module/ash/latest/ash-resource) using a\nconvenient DSL:\n\n```\ndefmodule MyApp.Factory do\n  use Smokestack\n\n  factory Character do\n    attribute :name, &Faker.StarWars.character/0\n    attribute :affiliation, choose([\"Galactic Empire\", \"Rebel Alliance\"])\n  end\nend\n\ndefmodule MyApp.CharacterTest do\n  use MyApp.DataCase\n  use MyApp.Factory\n\n  test \"it can build a character\" do\n    assert character = insert!(Character)\n  end\nend\n```","title":"Smokestack","ref":"Smokestack.html"},{"type":"behaviour","doc":"Each attribute uses a template to generate a value when building a factory.\nTemplates can be anything that implements the `Smokestack.Template` protocol.\nThis protocol is automatically implemented for functions with arities zero\nthrough two - meaning you can just drop in your own functions - or use one of\nthe built-in helpers from `Smokestack.Dsl.Template`.","title":"Templates - Smokestack","ref":"Smokestack.html#module-templates"},{"type":"behaviour","doc":"Sometimes you need to make slightly different factories to build a resource in\na specific state for your test scenario.\n\nHere's an example defining an alternate `:trek` variant for the character\nfactory defined above:\n\n```\nfactory Character, :trek do\n  attribute :name, choose([\"J.L. Pipes\", \"Severn\", \"Slickback\"])\n  attribute :affiliation, choose([\"Entrepreneur\", \"Voyager\"])\nend\n```","title":"Variants - Smokestack","ref":"Smokestack.html#module-variants"},{"type":"behaviour","doc":"You can use `insert/2` and `insert!/2` to build and insert records.  Records\nare inserted using `Ash.Seed.seed!/2`, which means that they bypass the usual\nAsh lifecycle (actions, validations, etc).","title":"Building resource records - Smokestack","ref":"Smokestack.html#module-building-resource-records"},{"type":"behaviour","doc":"- `load`: an atom, list of atoms or keyword list of the same listing\n  relationships, calculations and aggregates that should be loaded after the\n  record is created.\n- `count`: rather than inserting just a single record, you can specify a\n  number of records to be inserted.  A list of records will be returned.\n- `build`: an atom, list of atoms or keyword list of the same describing\n  relationships which you would like built alongside the record.  If the\n  related resource has a variant which matches the current one, it will be\n  used, and if not the `:default` variant will be.\n- `attrs`: A map or keyword list of attributes you would like to set directly\n  on the created record, rather than using the value provided by the factory.\n- `relate`: A keyword list of relationships to records (or lists of records)\n  to which you wish to directly relate the created record.","title":"Options - Smokestack","ref":"Smokestack.html#module-options"},{"type":"behaviour","doc":"As well as inserting records directly you can use `params/2` and `params!/2`\nto build parameters for use testing controller actions, HTTP requests, etc.","title":"Building parameters - Smokestack","ref":"Smokestack.html#module-building-parameters"},{"type":"behaviour","doc":"- `encode`: rather than returning a map or maps, provide an encoder module to\n  serialise the parameters.  Commonly you would use `Jason` or `Poison`.\n- `nest`: rather than returning a map or maps directly, wrap the result in an\n  outer map using the provided key.\n- `key_case`: change the case of the keys into one of the many cases supported\n  by [recase](https://hex.pm/packages/recase).\n- `key_type`: specify whether the returned map or maps should use string or\n  atom keys (ignored when using the `encode` option).\n- `count`: rather than returning just a single map, you can specify a number\n  of results to be returned.  A list of maps will be returned.\n- `build`: an atom, list of atoms or keyword list of the same describing\n  relationships which you would like built within the result.  If the related\n  resource has a variant which matches the current one, it will be used, and\n  if not the `:default` variant will be.\n- `attrs`: A map or keyword list of attributes you would like to set directly\n  on the result, rather than using the value provided by the factory.\n\n<!--- ash-hq-hide-start --> <!--- -->","title":"Options - Smokestack","ref":"Smokestack.html#module-options"},{"type":"behaviour","doc":"","title":"DSL Documentation - Smokestack","ref":"Smokestack.html#module-dsl-documentation"},{"type":"behaviour","doc":"* smokestack\n    * factory\n      * after_build\n      * attribute\n      * before_build","title":"Index - Smokestack","ref":"Smokestack.html#module-index"},{"type":"behaviour","doc":"","title":"Docs - Smokestack","ref":"Smokestack.html#module-docs"},{"type":"behaviour","doc":"* [factory](#module-factory)\n   * after_build\n   * attribute\n   * before_build\n\n\n\n\n\n---\n\n* `:domain` (module that adopts `Ash.Domain`) - The default Ash Domain to use when evaluating loads","title":"smokestack - Smokestack","ref":"Smokestack.html#module-smokestack"},{"type":"behaviour","doc":"Define factories for a resource\n\n    * after_build\n    * attribute\n    * before_build\n\n\n\n* `:domain` (module that adopts `Ash.Domain`) - The Ash Domain to use when evaluating loads\n\n* `:resource` (module that adopts `Ash.Resource`) - Required. An Ash Resource\n\n* `:variant` (`t:atom/0`) - The name of a factory variant The default value is `:default`.\n\n* `:auto_build` (one or a list of `t:atom/0`) - A list of relationships that should always be built when building this factory The default value is `[]`.\n\n* `:auto_load` - An Ash \"load statement\" to always apply when building this factory The default value is `[]`.\n\n\n\n##### after_build\n\nModify the record after building.\n\nAllows you to provide a function which can modify the built record before returning.\n\nThese hooks are only applied when building records and not parameters.\n\n\n\n\n\n\n* `:hook` (mfa or function of arity 1) - Required. A function which returns an updated record\n\n\n\n\n\n\n##### attribute\n\n\n\n\n\n\n\n* `:name` (`t:atom/0`) - Required. The name of the target attribute\n\n* `:generator` - Required. A function which can generate an appropriate value for the attribute.œ\n\n\n\n\n\n\n##### before_build\n\nModify the attributes before building.\n\nAllows you to provide a function which can modify the the attributes before building.\n\n\n\n\n\n\n* `:hook` (mfa or function of arity 1) - Required. A function which returns an updated record\n\n\n\n\n\n\n\n\n\n\n<!--- ash-hq-hide-stop --> <!--- -->","title":"factory - Smokestack","ref":"Smokestack.html#module-factory"},{"type":"behaviour","doc":"* `:extensions` (list of module that adopts `Spark.Dsl.Extension`) - A list of DSL extensions to add to the `Spark.Dsl`\n\n* `:otp_app` (`t:atom/0`) - The otp_app to use for any application configurable options\n\n* `:fragments` (list of `t:module/0`) - Fragments to include in the `Spark.Dsl`. See the fragments guide for more.","title":"Options - Smokestack","ref":"Smokestack.html#module-options"},{"type":"callback","doc":"Runs a factory and uses it to insert Ash resources into their data layers.\n\nAutomatically implemented by modules which `use Smokestack`.","title":"Smokestack.insert/2","ref":"Smokestack.html#c:insert/2"},{"type":"callback","doc":"* `:load` - An optional Ash load statement.  \n  You can request any calculations, aggregates or relationships you\n  would like loaded on the returned record.  \n  For example:  \n  ```elixir\n  insert!(Post, load: [:full_title])\n  assert is_binary(post.full_title)\n  ``` The default value is `[]`.\n\n* `:count` - Specify the number of instances to build.  \n  Use this option to run the factory a number of times and return the\n  results as a list.  \n  For example:  \n  ```elixir\n  posts = params!(Post, count: 3)\n  assert length(posts) == 3\n  ``` The default value is `nil`.\n\n* `:build` - A (nested) list of relationships to build.  \n  A (possibly nested) list of Ash resource relationships which is\n  traversed building any instances as needed.  \n  For example:  \n  ```elixir\n  post = insert!(Post, build: Author)\n  assert is_struct(post.author, Author)\n  ```  \n  Caveats:\n  - When building for a variant other than `:default` a matching\n    variant factory will be looked for and used if present, otherwise\n    it will build the default variant instead.  \n  - Note that for relationships whose cardinality is \"many\" we only\n    build one instance.  \n  If these caveats are an issue, then you can build them yourself and\n  pass them in using the `relate` option.  \n  For example:  \n  ```elixir\n  posts = insert!(Post, count: 3)\n  author = insert(Author, relate: [posts: posts])\n  ``` The default value is `[]`.\n\n* `:relate` (`t:keyword/0`) - A list of records to relate.  \n  For example  \n  ```elixir\n  author = insert!(Author)\n  post = insert!(Post, relate: [author: author])\n  ``` The default value is `[]`.\n\n* `:attrs` (`t:map/0`) - Attribute overrides.  \n  You can directly specify any overrides you would like set on the\n  resulting record without running their normal generator.  \n  For example:  \n  ```elixir\n  post = params!(Post, attrs: %{title: \"What's wrong with Huntly?\"})\n  assert post.title == \"What's wrong with Huntly?\"\n  ``` The default value is `%{}`.\n\nOptions for building relationships","title":"Options - Smokestack.insert/2","ref":"Smokestack.html#c:insert/2-options"},{"type":"callback","doc":"Raising version of `c:insert/2`.\n\nAutomatically implemented by modules which `use Smokestack`.","title":"Smokestack.insert!/2","ref":"Smokestack.html#c:insert!/2"},{"type":"callback","doc":"* `:load` - An optional Ash load statement.  \n  You can request any calculations, aggregates or relationships you\n  would like loaded on the returned record.  \n  For example:  \n  ```elixir\n  insert!(Post, load: [:full_title])\n  assert is_binary(post.full_title)\n  ``` The default value is `[]`.\n\n* `:count` - Specify the number of instances to build.  \n  Use this option to run the factory a number of times and return the\n  results as a list.  \n  For example:  \n  ```elixir\n  posts = params!(Post, count: 3)\n  assert length(posts) == 3\n  ``` The default value is `nil`.\n\n* `:build` - A (nested) list of relationships to build.  \n  A (possibly nested) list of Ash resource relationships which is\n  traversed building any instances as needed.  \n  For example:  \n  ```elixir\n  post = insert!(Post, build: Author)\n  assert is_struct(post.author, Author)\n  ```  \n  Caveats:\n  - When building for a variant other than `:default` a matching\n    variant factory will be looked for and used if present, otherwise\n    it will build the default variant instead.  \n  - Note that for relationships whose cardinality is \"many\" we only\n    build one instance.  \n  If these caveats are an issue, then you can build them yourself and\n  pass them in using the `relate` option.  \n  For example:  \n  ```elixir\n  posts = insert!(Post, count: 3)\n  author = insert(Author, relate: [posts: posts])\n  ``` The default value is `[]`.\n\n* `:relate` (`t:keyword/0`) - A list of records to relate.  \n  For example  \n  ```elixir\n  author = insert!(Author)\n  post = insert!(Post, relate: [author: author])\n  ``` The default value is `[]`.\n\n* `:attrs` (`t:map/0`) - Attribute overrides.  \n  You can directly specify any overrides you would like set on the\n  resulting record without running their normal generator.  \n  For example:  \n  ```elixir\n  post = params!(Post, attrs: %{title: \"What's wrong with Huntly?\"})\n  assert post.title == \"What's wrong with Huntly?\"\n  ``` The default value is `%{}`.\n\nOptions for building relationships","title":"Options - Smokestack.insert!/2","ref":"Smokestack.html#c:insert!/2-options"},{"type":"type","doc":"","title":"Smokestack.insert_option/0","ref":"Smokestack.html#t:insert_option/0"},{"type":"type","doc":"","title":"Smokestack.param_option/0","ref":"Smokestack.html#t:param_option/0"},{"type":"callback","doc":"Runs a factory and uses it to build a parameters suitable for simulating a\nrequest.\n\nAutomatically implemented by modules which `use Smokestack`.","title":"Smokestack.params/2","ref":"Smokestack.html#c:params/2"},{"type":"callback","doc":"* `:encode` - Provide an encoder module.  \n  If provided the result will be passed to an `encode/1` function\n  which should return an ok/error tuple with the encoded result.  \n  This is primarily for use with `Jason` or `Poison`, however you may\n  wish to define your own encoder for your tests.  \n  For example:  \n  ```elixir\n  post = params!(Post, encoder: Jason)\n  assert Jason.decode!(post)\n  ```\n\n* `:nest` - Nest the result within an arbitrary map key.  \n  Mostly provided as a shorthand for building API requests where the\n  built instance may need to be nested inside a key such as `data`.  \n  Takes the result and nests it in a map under the provided key.  \n  For example:  \n  ```elixir\n  request = params!(Post, nest: :data)\n  assert is_binary(request.data.title)\n  ```  \n  If you need a more complicated mangling of the result, I suggest\n  using the `encode` option.\n\n* `:key_case` - Change the key case.  \n  Sometimes you will need to change the case of the keys before sending\n  to an API.  Behind the scenes we use [recase](https://hexdocs.pm/recase)\n  to remap the keys as specified.  \n  For example:  \n      iex> params!(Post, key_case: :kebab) |> Map.keys()\n      [:title, :\"sub-title\"] The default value is `:snake`.\n\n* `:key_type` - Specify string or atom keys.  \n  Allows you to specify the type of the returned keys.  \n  For example:  \n      iex> params!(Post, key_type: :string) |> Map.keys()\n      [\"title\", \"sub_title\"]\n   Valid values are :string, :atom The default value is `:atom`.\n\n* `:count` - Specify the number of instances to build.  \n  Use this option to run the factory a number of times and return the\n  results as a list.  \n  For example:  \n  ```elixir\n  posts = params!(Post, count: 3)\n  assert length(posts) == 3\n  ``` The default value is `nil`.\n\n* `:build` - A (nested) list of relationships to build.  \n  A (possibly nested) list of Ash resource relationships which is\n  traversed building any instances as needed.  \n  For example:  \n  ```elixir\n  post = insert!(Post, build: Author)\n  assert is_struct(post.author, Author)\n  ```  \n  Caveats:\n  - When building for a variant other than `:default` a matching\n    variant factory will be looked for and used if present, otherwise\n    it will build the default variant instead.  \n  - Note that for relationships whose cardinality is \"many\" we only\n    build one instance.  \n  If these caveats are an issue, then you can build them yourself and\n  pass them in using the `relate` option.  \n  For example:  \n  ```elixir\n  posts = insert!(Post, count: 3)\n  author = insert(Author, relate: [posts: posts])\n  ``` The default value is `[]`.\n\n* `:relate` (`t:keyword/0`) - A list of records to relate.  \n  For example  \n  ```elixir\n  author = insert!(Author)\n  post = insert!(Post, relate: [author: author])\n  ``` The default value is `[]`.\n\n* `:attrs` (`t:map/0`) - Attribute overrides.  \n  You can directly specify any overrides you would like set on the\n  resulting record without running their normal generator.  \n  For example:  \n  ```elixir\n  post = params!(Post, attrs: %{title: \"What's wrong with Huntly?\"})\n  assert post.title == \"What's wrong with Huntly?\"\n  ``` The default value is `%{}`.\n\nOptions for building relationships","title":"Options - Smokestack.params/2","ref":"Smokestack.html#c:params/2-options"},{"type":"callback","doc":"Raising version of `c:params/2`.\n\nAutomatically implemented by modules which `use Smokestack`.","title":"Smokestack.params!/2","ref":"Smokestack.html#c:params!/2"},{"type":"callback","doc":"* `:encode` - Provide an encoder module.  \n  If provided the result will be passed to an `encode/1` function\n  which should return an ok/error tuple with the encoded result.  \n  This is primarily for use with `Jason` or `Poison`, however you may\n  wish to define your own encoder for your tests.  \n  For example:  \n  ```elixir\n  post = params!(Post, encoder: Jason)\n  assert Jason.decode!(post)\n  ```\n\n* `:nest` - Nest the result within an arbitrary map key.  \n  Mostly provided as a shorthand for building API requests where the\n  built instance may need to be nested inside a key such as `data`.  \n  Takes the result and nests it in a map under the provided key.  \n  For example:  \n  ```elixir\n  request = params!(Post, nest: :data)\n  assert is_binary(request.data.title)\n  ```  \n  If you need a more complicated mangling of the result, I suggest\n  using the `encode` option.\n\n* `:key_case` - Change the key case.  \n  Sometimes you will need to change the case of the keys before sending\n  to an API.  Behind the scenes we use [recase](https://hexdocs.pm/recase)\n  to remap the keys as specified.  \n  For example:  \n      iex> params!(Post, key_case: :kebab) |> Map.keys()\n      [:title, :\"sub-title\"] The default value is `:snake`.\n\n* `:key_type` - Specify string or atom keys.  \n  Allows you to specify the type of the returned keys.  \n  For example:  \n      iex> params!(Post, key_type: :string) |> Map.keys()\n      [\"title\", \"sub_title\"]\n   Valid values are :string, :atom The default value is `:atom`.\n\n* `:count` - Specify the number of instances to build.  \n  Use this option to run the factory a number of times and return the\n  results as a list.  \n  For example:  \n  ```elixir\n  posts = params!(Post, count: 3)\n  assert length(posts) == 3\n  ``` The default value is `nil`.\n\n* `:build` - A (nested) list of relationships to build.  \n  A (possibly nested) list of Ash resource relationships which is\n  traversed building any instances as needed.  \n  For example:  \n  ```elixir\n  post = insert!(Post, build: Author)\n  assert is_struct(post.author, Author)\n  ```  \n  Caveats:\n  - When building for a variant other than `:default` a matching\n    variant factory will be looked for and used if present, otherwise\n    it will build the default variant instead.  \n  - Note that for relationships whose cardinality is \"many\" we only\n    build one instance.  \n  If these caveats are an issue, then you can build them yourself and\n  pass them in using the `relate` option.  \n  For example:  \n  ```elixir\n  posts = insert!(Post, count: 3)\n  author = insert(Author, relate: [posts: posts])\n  ``` The default value is `[]`.\n\n* `:relate` (`t:keyword/0`) - A list of records to relate.  \n  For example  \n  ```elixir\n  author = insert!(Author)\n  post = insert!(Post, relate: [author: author])\n  ``` The default value is `[]`.\n\n* `:attrs` (`t:map/0`) - Attribute overrides.  \n  You can directly specify any overrides you would like set on the\n  resulting record without running their normal generator.  \n  For example:  \n  ```elixir\n  post = params!(Post, attrs: %{title: \"What's wrong with Huntly?\"})\n  assert post.title == \"What's wrong with Huntly?\"\n  ``` The default value is `%{}`.\n\nOptions for building relationships","title":"Options - Smokestack.params!/2","ref":"Smokestack.html#c:params!/2-options"},{"type":"type","doc":"","title":"Smokestack.recursive_atom_list/0","ref":"Smokestack.html#t:recursive_atom_list/0"},{"type":"type","doc":"","title":"Smokestack.t/0","ref":"Smokestack.html#t:t/0"},{"type":"type","doc":"Choose a factory variant to use.  Defaults to `:default`.","title":"Smokestack.variant_option/0","ref":"Smokestack.html#t:variant_option/0"},{"type":"behaviour","doc":"A generic behaviour for \"building things\".","title":"Smokestack.Builder","ref":"Smokestack.Builder.html"},{"type":"callback","doc":"Given a Factory entity and some options build something.","title":"Smokestack.Builder.build/2","ref":"Smokestack.Builder.html#c:build/2"},{"type":"function","doc":"Find the appropriate factory, validate options and run the builder.","title":"Smokestack.Builder.build/4","ref":"Smokestack.Builder.html#build/4"},{"type":"function","doc":"Generate documentation for the available options.","title":"Smokestack.Builder.docs/2","ref":"Smokestack.Builder.html#docs/2"},{"type":"type","doc":"","title":"Smokestack.Builder.error/0","ref":"Smokestack.Builder.html#t:error/0"},{"type":"callback","doc":"Provide a schema for validating options.","title":"Smokestack.Builder.option_schema/1","ref":"Smokestack.Builder.html#c:option_schema/1"},{"type":"type","doc":"","title":"Smokestack.Builder.result/0","ref":"Smokestack.Builder.html#t:result/0"},{"type":"type","doc":"","title":"Smokestack.Builder.t/0","ref":"Smokestack.Builder.html#t:t/0"},{"type":"module","doc":"The DSL definition for the Smokestack DSL.\n\n<!--- ash-hq-hide-start --> <!--- -->","title":"Smokestack.Dsl","ref":"Smokestack.Dsl.html"},{"type":"module","doc":"","title":"DSL Documentation - Smokestack.Dsl","ref":"Smokestack.Dsl.html#module-dsl-documentation"},{"type":"module","doc":"* smokestack\n    * factory\n      * after_build\n      * attribute\n      * before_build","title":"Index - Smokestack.Dsl","ref":"Smokestack.Dsl.html#module-index"},{"type":"module","doc":"","title":"Docs - Smokestack.Dsl","ref":"Smokestack.Dsl.html#module-docs"},{"type":"module","doc":"* [factory](#module-factory)\n   * after_build\n   * attribute\n   * before_build\n\n\n\n\n\n---\n\n* `:domain` (module that adopts `Ash.Domain`) - The default Ash Domain to use when evaluating loads","title":"smokestack - Smokestack.Dsl","ref":"Smokestack.Dsl.html#module-smokestack"},{"type":"module","doc":"Define factories for a resource\n\n    * after_build\n    * attribute\n    * before_build\n\n\n\n* `:domain` (module that adopts `Ash.Domain`) - The Ash Domain to use when evaluating loads\n\n* `:resource` (module that adopts `Ash.Resource`) - Required. An Ash Resource\n\n* `:variant` (`t:atom/0`) - The name of a factory variant The default value is `:default`.\n\n* `:auto_build` (one or a list of `t:atom/0`) - A list of relationships that should always be built when building this factory The default value is `[]`.\n\n* `:auto_load` - An Ash \"load statement\" to always apply when building this factory The default value is `[]`.\n\n\n\n##### after_build\n\nModify the record after building.\n\nAllows you to provide a function which can modify the built record before returning.\n\nThese hooks are only applied when building records and not parameters.\n\n\n\n\n\n\n* `:hook` (mfa or function of arity 1) - Required. A function which returns an updated record\n\n\n\n\n\n\n##### attribute\n\n\n\n\n\n\n\n* `:name` (`t:atom/0`) - Required. The name of the target attribute\n\n* `:generator` - Required. A function which can generate an appropriate value for the attribute.œ\n\n\n\n\n\n\n##### before_build\n\nModify the attributes before building.\n\nAllows you to provide a function which can modify the the attributes before building.\n\n\n\n\n\n\n* `:hook` (mfa or function of arity 1) - Required. A function which returns an updated record\n\n\n\n\n\n\n\n\n\n\n<!--- ash-hq-hide-stop --> <!--- -->","title":"factory - Smokestack.Dsl","ref":"Smokestack.Dsl.html#module-factory"},{"type":"module","doc":"The `after_build` DSL entity.\n\nSee `d:Smokestack.factory.after_build` for more information.","title":"Smokestack.Dsl.AfterBuild","ref":"Smokestack.Dsl.AfterBuild.html"},{"type":"type","doc":"","title":"Smokestack.Dsl.AfterBuild.t/0","ref":"Smokestack.Dsl.AfterBuild.html#t:t/0"},{"type":"module","doc":"The `attribute ` DSL entity.\n\nSee `d:Smokestack.factory.attribute` for more information.","title":"Smokestack.Dsl.Attribute","ref":"Smokestack.Dsl.Attribute.html"},{"type":"type","doc":"","title":"Smokestack.Dsl.Attribute.t/0","ref":"Smokestack.Dsl.Attribute.html#t:t/0"},{"type":"module","doc":"The `before_build` DSL entity.\n\nSee `d:Smokestack.factory.before_build` for more information.","title":"Smokestack.Dsl.BeforeBuild","ref":"Smokestack.Dsl.BeforeBuild.html"},{"type":"type","doc":"","title":"Smokestack.Dsl.BeforeBuild.attrs/0","ref":"Smokestack.Dsl.BeforeBuild.html#t:attrs/0"},{"type":"type","doc":"","title":"Smokestack.Dsl.BeforeBuild.t/0","ref":"Smokestack.Dsl.BeforeBuild.html#t:t/0"},{"type":"module","doc":"The `factory` DSL entity.\n\nSee `d:Smokestack.factory` for more information.","title":"Smokestack.Dsl.Factory","ref":"Smokestack.Dsl.Factory.html"},{"type":"type","doc":"","title":"Smokestack.Dsl.Factory.t/0","ref":"Smokestack.Dsl.Factory.html#t:t/0"},{"type":"module","doc":"Introspection of Smokestack DSLs.","title":"Smokestack.Dsl.Info","ref":"Smokestack.Dsl.Info.html"},{"type":"function","doc":"Retrieve a variant for a specific resource.","title":"Smokestack.Dsl.Info.factory/3","ref":"Smokestack.Dsl.Info.html#factory/3"},{"type":"function","doc":"Raising version of `factory/3`","title":"Smokestack.Dsl.Info.factory!/3","ref":"Smokestack.Dsl.Info.html#factory!/3"},{"type":"function","doc":"List all variants available for a resource.","title":"Smokestack.Dsl.Info.variants/2","ref":"Smokestack.Dsl.Info.html#variants/2"},{"type":"module","doc":"Templates which assist in the generation of values.œ","title":"Smokestack.Dsl.Template","ref":"Smokestack.Dsl.Template.html"},{"type":"function","doc":"Randomly select between a list of options.","title":"Smokestack.Dsl.Template.choose/2","ref":"Smokestack.Dsl.Template.html#choose/2"},{"type":"function","doc":"Select a constant value","title":"Smokestack.Dsl.Template.constant/2","ref":"Smokestack.Dsl.Template.html#constant/2"},{"type":"function","doc":"Cycle sequentially between a list of options.","title":"Smokestack.Dsl.Template.cycle/2","ref":"Smokestack.Dsl.Template.html#cycle/2"},{"type":"type","doc":"","title":"Smokestack.Dsl.Template.element/0","ref":"Smokestack.Dsl.Template.html#t:element/0"},{"type":"type","doc":"","title":"Smokestack.Dsl.Template.mapper/0","ref":"Smokestack.Dsl.Template.html#t:mapper/0"},{"type":"function","doc":"Call a generator a number of times.","title":"Smokestack.Dsl.Template.n_times/3","ref":"Smokestack.Dsl.Template.html#n_times/3"},{"type":"function","doc":"Generate sequential values.","title":"Smokestack.Dsl.Template.sequence/2","ref":"Smokestack.Dsl.Template.html#sequence/2"},{"type":"module","doc":"Executes a factory and returns it's result.","title":"Smokestack.FactoryBuilder","ref":"Smokestack.FactoryBuilder.html"},{"type":"type","doc":"Manually specify some attributes.","title":"Smokestack.FactoryBuilder.attrs_option/0","ref":"Smokestack.FactoryBuilder.html#t:attrs_option/0"},{"type":"function","doc":"Execute the named factory, if possible.","title":"Smokestack.FactoryBuilder.build/2","ref":"Smokestack.FactoryBuilder.html#build/2"},{"type":"type","doc":"","title":"Smokestack.FactoryBuilder.error/0","ref":"Smokestack.FactoryBuilder.html#t:error/0"},{"type":"type","doc":"","title":"Smokestack.FactoryBuilder.option/0","ref":"Smokestack.FactoryBuilder.html#t:option/0"},{"type":"type","doc":"","title":"Smokestack.FactoryBuilder.result/0","ref":"Smokestack.FactoryBuilder.html#t:result/0"},{"type":"module","doc":"Handles repeatedly building.","title":"Smokestack.ManyBuilder","ref":"Smokestack.ManyBuilder.html"},{"type":"function","doc":"Run the factory a number of times.","title":"Smokestack.ManyBuilder.build/2","ref":"Smokestack.ManyBuilder.html#build/2"},{"type":"type","doc":"How many times should we call the builder?","title":"Smokestack.ManyBuilder.count_option/0","ref":"Smokestack.ManyBuilder.html#t:count_option/0"},{"type":"type","doc":"","title":"Smokestack.ManyBuilder.error/0","ref":"Smokestack.ManyBuilder.html#t:error/0"},{"type":"type","doc":"","title":"Smokestack.ManyBuilder.option/0","ref":"Smokestack.ManyBuilder.html#t:option/0"},{"type":"type","doc":"","title":"Smokestack.ManyBuilder.result/0","ref":"Smokestack.ManyBuilder.html#t:result/0"},{"type":"module","doc":"Handles the building of parameters.","title":"Smokestack.ParamBuilder","ref":"Smokestack.ParamBuilder.html"},{"type":"function","doc":"Run the factory and return a map or list-of-maps of params.","title":"Smokestack.ParamBuilder.build/2","ref":"Smokestack.ParamBuilder.html#build/2"},{"type":"type","doc":"Encode the result using the specified encoder.\n\nSmokestack will call `encode/1` on the module provided with the generated\nresult (or results).  For example set `encode: Jason` or `encode: Poison`\nto encode the results as a JSON string.\n\nThe module's encode function should return an ok/error tuple.","title":"Smokestack.ParamBuilder.encode_option/0","ref":"Smokestack.ParamBuilder.html#t:encode_option/0"},{"type":"type","doc":"","title":"Smokestack.ParamBuilder.error/0","ref":"Smokestack.ParamBuilder.html#t:error/0"},{"type":"type","doc":"Format the keys in the specified case. Defaults to `:snake`","title":"Smokestack.ParamBuilder.key_case_option/0","ref":"Smokestack.ParamBuilder.html#t:key_case_option/0"},{"type":"type","doc":"Convert the keys into the specified type.  Defaults to `:atom`.","title":"Smokestack.ParamBuilder.key_type_option/0","ref":"Smokestack.ParamBuilder.html#t:key_type_option/0"},{"type":"type","doc":"Nest the result within the specified key in the output.","title":"Smokestack.ParamBuilder.nest_option/0","ref":"Smokestack.ParamBuilder.html#t:nest_option/0"},{"type":"type","doc":"","title":"Smokestack.ParamBuilder.option/0","ref":"Smokestack.ParamBuilder.html#t:option/0"},{"type":"type","doc":"","title":"Smokestack.ParamBuilder.result/0","ref":"Smokestack.ParamBuilder.html#t:result/0"},{"type":"module","doc":"Handles the insertion of new records.","title":"Smokestack.RecordBuilder","ref":"Smokestack.RecordBuilder.html"},{"type":"function","doc":"Run the factory and insert a record, or records.","title":"Smokestack.RecordBuilder.build/2","ref":"Smokestack.RecordBuilder.html#build/2"},{"type":"type","doc":"","title":"Smokestack.RecordBuilder.error/0","ref":"Smokestack.RecordBuilder.html#t:error/0"},{"type":"type","doc":"A nested keyword list of associations, calculations and aggregates to load","title":"Smokestack.RecordBuilder.load_option/0","ref":"Smokestack.RecordBuilder.html#t:load_option/0"},{"type":"type","doc":"","title":"Smokestack.RecordBuilder.option/0","ref":"Smokestack.RecordBuilder.html#t:option/0"},{"type":"type","doc":"","title":"Smokestack.RecordBuilder.result/0","ref":"Smokestack.RecordBuilder.html#t:result/0"},{"type":"module","doc":"Recursively build the factory and any related factories that have been\nrequested.","title":"Smokestack.RelatedBuilder","ref":"Smokestack.RelatedBuilder.html"},{"type":"function","doc":"Build related factories, if required.","title":"Smokestack.RelatedBuilder.build/2","ref":"Smokestack.RelatedBuilder.html#build/2"},{"type":"type","doc":"A nested keyword list of associations that should also be built.","title":"Smokestack.RelatedBuilder.build_option/0","ref":"Smokestack.RelatedBuilder.html#t:build_option/0"},{"type":"type","doc":"","title":"Smokestack.RelatedBuilder.error/0","ref":"Smokestack.RelatedBuilder.html#t:error/0"},{"type":"type","doc":"","title":"Smokestack.RelatedBuilder.option/0","ref":"Smokestack.RelatedBuilder.html#t:option/0"},{"type":"type","doc":"A nested keyword list of previously built records to explicitly associate to the new record.","title":"Smokestack.RelatedBuilder.relate_option/0","ref":"Smokestack.RelatedBuilder.html#t:relate_option/0"},{"type":"type","doc":"","title":"Smokestack.RelatedBuilder.result/0","ref":"Smokestack.RelatedBuilder.html#t:result/0"},{"type":"protocol","doc":"A protocol for generating values from templates.","title":"Smokestack.Template","ref":"Smokestack.Template.html"},{"type":"function","doc":"Generate a value from the template.","title":"Smokestack.Template.generate/3","ref":"Smokestack.Template.html#generate/3"},{"type":"function","doc":"Initialise the template, if required.","title":"Smokestack.Template.init/1","ref":"Smokestack.Template.html#init/1"},{"type":"type","doc":"","title":"Smokestack.Template.mapper/0","ref":"Smokestack.Template.html#t:mapper/0"},{"type":"type","doc":"All the types that implement this protocol.","title":"Smokestack.Template.t/0","ref":"Smokestack.Template.html#t:t/0"},{"type":"extras","doc":"# Smokestack\n\n[![Build Status](https://drone.harton.dev/api/badges/james/smokestack/status.svg?ref=refs/heads/main)](https://drone.harton.nz/james/smokestack)\n[![Hippocratic License HL3-FULL](https://img.shields.io/static/v1?label=Hippocratic%20License&message=HL3-FULL&labelColor=5e2751&color=bc8c3d)](https://firstdonoharm.dev/version/3/0/full.html)\n\nSmokestack provides a way to define test factories for your [Ash Resources](https://ash-hq.org/docs/module/ash/latest/ash-resource) using a convenient DSL:\n\n```elixir\ndefmodule MyApp.Factory do\n  use Smokestack\n\n  factory Character do\n    attribute :name, &Faker.StarWars.character/0\n    attribute :affiliation, choose([\"Galactic Empire\", \"Rebel Alliance\"])\n  end\n\n  factory Character, :trek do\n    attribute :name, choose([\"J.L. Pipes\", \"Severn\", \"Slickback\"])\n    attribute :affiliation, choose([\"Entrepreneur\", \"Voyager\"])\n  end\nend\n\ndefmodule MyApp.CharacterTest do\n  use MyApp.DataCase\n  use MyApp.Factory\n\n  test \"it can build a character\" do\n    assert character = insert!(Character)\n  end\nend\n```","title":"Smokestack","ref":"readme.html"},{"type":"extras","doc":"Smokestack is available on [Hex](https://hex.pm/packages/smokestack) you can\nadd it directly to your `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:smokestack, \"~> 0.9.2\"},\n  ]\nend\n```\n\nDocumentation for the latest release is available on [HexDocs](http://hexdocs.pm/smokestack).","title":"Installation - Smokestack","ref":"readme.html#installation"},{"type":"extras","doc":"This repository is mirrored [on Github](https://github.com/jimsynz/smokestack)\nfrom it's primary location [on my Forgejo instance](https://harton.dev/james/smokestack).\nFeel free to raise issues and open PRs on Github.","title":"Github Mirror - Smokestack","ref":"readme.html#github-mirror"},{"type":"extras","doc":"This software is licensed under the terms of the\n[HL3-FULL](https://firstdonoharm.dev), see the `LICENSE.md` file included with\nthis package for the terms.\n\nThis license actively proscribes this software being used by and for some\nindustries, countries and activities. If your usage of this software doesn't\ncomply with the terms of this license, then [contact me](mailto:james@harton.nz)\nwith the details of your use-case to organise the purchase of a license - the\ncost of which may include a donation to a suitable charity or NGO.","title":"License - Smokestack","ref":"readme.html#license"},{"type":"extras","doc":"# Change Log\n\nAll notable changes to this project will be documented in this file.\nSee [Conventional Commits](Https://conventionalcommits.org) for commit guidelines.\n\n<!-- changelog -->\n\n## [v0.9.2](https://harton.dev/james/smokestack/compare/v0.9.1...v0.9.2) (2025-01-30)","title":"Change Log","ref":"changelog.html"},{"type":"extras","doc":"* Prevent the `relate` option to trickle down to descendants when building\n\n## [v0.9.1](https://harton.dev/james/smokestack/compare/v0.9.0...v0.9.1) (2024-05-30)","title":"Bug Fixes: - Change Log","ref":"changelog.html#bug-fixes"},{"type":"extras","doc":"* allow auto builds to be replaced by explicit relate commands.\n\n## [v0.9.0](https://harton.dev/james/smokestack/compare/v0.8.1...v0.9.0) (2024-05-28)","title":"Bug Fixes: - Change Log","ref":"changelog.html#bug-fixes"},{"type":"extras","doc":"* Auto build/load factory options. (#83)\n\n## [v0.8.1](https://harton.dev/james/smokestack/compare/v0.8.0...v0.8.1) (2024-05-28)","title":"Features: - Change Log","ref":"changelog.html#features"},{"type":"extras","doc":"* Include `:variant` in option schema.\n\n* bug with generator arguments.\n\n## [v0.8.0](https://harton.dev/james/smokestack/compare/v0.7.0...v0.8.0) (2024-05-26)","title":"Bug Fixes: - Change Log","ref":"changelog.html#bug-fixes"},{"type":"extras","doc":"* Add `before_build` and `after_build` entities to factories.\n\n## [v0.7.0](https://harton.dev/james/smokestack/compare/v0.6.2...v0.7.0) (2024-05-20)","title":"Features: - Change Log","ref":"changelog.html#features"},{"type":"extras","doc":"* related: Newly build records can now be related to existing records.\n\n## [v0.6.2](https://harton.dev/james/smokestack/compare/v0.6.1-rc.2...v0.6.2) (2024-05-11)\n\n\n\n\n## [v0.6.1-rc.2](https://harton.dev/james/smokestack/compare/v0.6.1-rc.1...v0.6.1-rc.2) (2024-04-03)","title":"Features: - Change Log","ref":"changelog.html#features"},{"type":"extras","doc":"* loosen Ash RC requirement.\n\n## [v0.6.1-rc.1](https://harton.dev/james/smokestack/compare/v0.6.1-rc.0...v0.6.1-rc.1) (2024-04-01)","title":"Improvements: - Change Log","ref":"changelog.html#improvements"},{"type":"extras","doc":"* docs reference to `Ash.Seed.seed!/2`.\n\n## [v0.6.1-rc.0](https://harton.dev/james/smokestack/compare/v0.6.0...v0.6.1-rc.0) (2024-03-30)\n\n\n\n\n## [v0.6.0](https://harton.dev/james/smokestack/compare/v0.5.0-rc.0...v0.6.0) (2024-03-27)\n\n\n\n\n## [v0.5.0-rc.0](https://harton.dev/james/smokestack/compare/v0.4.2...v0.5.0-rc.0) (2024-03-27)","title":"Bug Fixes: - Change Log","ref":"changelog.html#bug-fixes"},{"type":"extras","doc":"* Ash 3.0 support. (#42)\n\n\n\n## [v0.4.2](https://harton.dev/james/smokestack/compare/v0.4.1...v0.4.2) (2024-03-22)","title":"Breaking Changes: - Change Log","ref":"changelog.html#breaking-changes"},{"type":"extras","doc":"* Fixes inability to `use` generated factories.\n\n## [v0.4.1](https://harton.dev/james/smokestack/compare/v0.4.0...v0.4.1) (2023-09-20)","title":"Bug Fixes: - Change Log","ref":"changelog.html#bug-fixes"},{"type":"extras","doc":"* bug in constant mapper.\n\n## [v0.4.0](https://harton.dev/james/smokestack/compare/v0.3.1...v0.4.0) (2023-09-19)","title":"Bug Fixes: - Change Log","ref":"changelog.html#bug-fixes"},{"type":"extras","doc":"* add template for constant values\n\n## [v0.3.1](https://harton.dev/james/smokestack/compare/v0.3.0...v0.3.1) (2023-09-09)\n\n\n\n\n## [v0.3.0](https://harton.dev/james/smokestack/compare/v0.2.0...v0.3.0) (2023-09-05)","title":"Features: - Change Log","ref":"changelog.html#features"},{"type":"extras","doc":"* build related parameters when requested. (#4)\n\n## [v0.2.0](https://harton.dev/james/smokestack/compare/v0.1.1...v0.2.0) (2023-09-05)","title":"Features: - Change Log","ref":"changelog.html#features"},{"type":"extras","doc":"* build related parameters when requested. (#4)\n\n## [v0.1.1](https://harton.dev/james/smokestack/compare/v0.1.0...v0.1.1) (2023-08-18)","title":"Features: - Change Log","ref":"changelog.html#features"},{"type":"extras","doc":"* Add `insert_many`.\n\n* Add `Smokestack` behaviour.\n\n* Add `Builder.insert/2..5`.\n\n## [v0.1.0](https://harton.dev/james/smokestack/compare/v0.1.0...v0.1.0) (2023-08-16)","title":"Improvements: - Change Log","ref":"changelog.html#improvements"},{"type":"extras","doc":"* Factory DSL and param building.","title":"Features: - Change Log","ref":"changelog.html#features"},{"type":"extras","doc":"<!--\nThis file was generated by Spark. Do not edit it by hand.\n-->\n# DSL: Smokestack.Dsl\n\nThe DSL definition for the Smokestack DSL.\n\n<!--- ash-hq-hide-start --> <!--- -->","title":"DSL: Smokestack.Dsl","ref":"dsl-smokestack.html"},{"type":"extras","doc":"","title":"DSL Documentation - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#dsl-documentation"},{"type":"extras","doc":"* smokestack\n    * factory\n      * after_build\n      * attribute\n      * before_build","title":"Index - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#index"},{"type":"extras","doc":"","title":"Docs - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#docs"},{"type":"extras","doc":"* [factory](#module-factory)\n   * after_build\n   * attribute\n   * before_build\n\n\n\n\n\n---\n\n* `:domain` (module that adopts `Ash.Domain`) - The default Ash Domain to use when evaluating loads","title":"smokestack - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#smokestack"},{"type":"extras","doc":"Define factories for a resource\n\n    * after_build\n    * attribute\n    * before_build\n\n\n\n* `:domain` (module that adopts `Ash.Domain`) - The Ash Domain to use when evaluating loads\n\n* `:resource` (module that adopts `Ash.Resource`) - Required. An Ash Resource\n\n* `:variant` (`t:atom/0`) - The name of a factory variant The default value is `:default`.\n\n* `:auto_build` (one or a list of `t:atom/0`) - A list of relationships that should always be built when building this factory The default value is `[]`.\n\n* `:auto_load` - An Ash \"load statement\" to always apply when building this factory The default value is `[]`.\n\n\n\n##### after_build\n\nModify the record after building.\n\nAllows you to provide a function which can modify the built record before returning.\n\nThese hooks are only applied when building records and not parameters.\n\n\n\n\n\n\n* `:hook` (mfa or function of arity 1) - Required. A function which returns an updated record\n\n\n\n\n\n\n##### attribute\n\n\n\n\n\n\n\n* `:name` (`t:atom/0`) - Required. The name of the target attribute\n\n* `:generator` - Required. A function which can generate an appropriate value for the attribute.œ\n\n\n\n\n\n\n##### before_build\n\nModify the attributes before building.\n\nAllows you to provide a function which can modify the the attributes before building.\n\n\n\n\n\n\n* `:hook` (mfa or function of arity 1) - Required. A function which returns an updated record\n\n\n\n\n\n\n\n\n\n\n<!--- ash-hq-hide-stop --> <!--- -->","title":"factory - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#factory"},{"type":"extras","doc":"","title":"smokestack - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#smokestack"},{"type":"extras","doc":"* [factory](#smokestack-factory)\n   * after_build\n   * attribute\n   * before_build","title":"Nested DSLs - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#nested-dsls"},{"type":"extras","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`domain`](#smokestack-domain){: #smokestack-domain } | `module` |  | The default Ash Domain to use when evaluating loads |","title":"Options - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#options"},{"type":"extras","doc":"```elixir\nfactory resource, variant \\\\ :default\n```\n\n\nDefine factories for a resource","title":"smokestack.factory - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#smokestack-factory"},{"type":"extras","doc":"* [after_build](#smokestack-factory-after_build)\n * [attribute](#smokestack-factory-attribute)\n * [before_build](#smokestack-factory-before_build)","title":"Nested DSLs - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#nested-dsls"},{"type":"extras","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`resource`](#smokestack-factory-resource){: #smokestack-factory-resource .spark-required} | `module` |  | An Ash Resource |\n| [`variant`](#smokestack-factory-variant){: #smokestack-factory-variant } | `atom` | `:default` | The name of a factory variant |","title":"Arguments - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#arguments"},{"type":"extras","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`domain`](#smokestack-factory-domain){: #smokestack-factory-domain } | `module` |  | The Ash Domain to use when evaluating loads |\n| [`auto_build`](#smokestack-factory-auto_build){: #smokestack-factory-auto_build } | `atom \\| list(atom)` | `[]` | A list of relationships that should always be built when building this factory |\n| [`auto_load`](#smokestack-factory-auto_load){: #smokestack-factory-auto_load } | `atom \\| keyword \\| list(atom \\| keyword)` | `[]` | An Ash \"load statement\" to always apply when building this factory |","title":"Options - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#options"},{"type":"extras","doc":"```elixir\nafter_build hook\n```\n\n\nModify the record after building.\n\nAllows you to provide a function which can modify the built record before returning.\n\nThese hooks are only applied when building records and not parameters.","title":"smokestack.factory.after_build - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#smokestack-factory-after_build"},{"type":"extras","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`hook`](#smokestack-factory-after_build-hook){: #smokestack-factory-after_build-hook .spark-required} | `(any -> any) \\| mfa` |  | A function which returns an updated record |","title":"Arguments - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#arguments"},{"type":"extras","doc":"Target: `Smokestack.Dsl.AfterBuild`","title":"Introspection - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#introspection"},{"type":"extras","doc":"```elixir\nattribute name, generator\n```","title":"smokestack.factory.attribute - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#smokestack-factory-attribute"},{"type":"extras","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`name`](#smokestack-factory-attribute-name){: #smokestack-factory-attribute-name .spark-required} | `atom` |  | The name of the target attribute |\n| [`generator`](#smokestack-factory-attribute-generator){: #smokestack-factory-attribute-generator .spark-required} | `(-> any) \\| mfa \\| (any -> any) \\| mfa \\| (any, any -> any) \\| mfa \\| Smokestack.Template.Choose \\| Smokestack.Template.Constant \\| Smokestack.Template.Cycle \\| Smokestack.Template.NTimes \\| Smokestack.Template.Sequence` |  | A function which can generate an appropriate value for the attribute.œ |","title":"Arguments - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#arguments"},{"type":"extras","doc":"Target: `Smokestack.Dsl.Attribute`","title":"Introspection - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#introspection"},{"type":"extras","doc":"```elixir\nbefore_build hook\n```\n\n\nModify the attributes before building.\n\nAllows you to provide a function which can modify the the attributes before building.","title":"smokestack.factory.before_build - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#smokestack-factory-before_build"},{"type":"extras","doc":"| Name | Type | Default | Docs |\n|------|------|---------|------|\n| [`hook`](#smokestack-factory-before_build-hook){: #smokestack-factory-before_build-hook .spark-required} | `(any -> any) \\| mfa` |  | A function which returns an updated record |","title":"Arguments - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#arguments"},{"type":"extras","doc":"Target: `Smokestack.Dsl.BeforeBuild`","title":"Introspection - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#introspection"},{"type":"extras","doc":"Target: `Smokestack.Dsl.Factory`\n\n\n\n\n\n .spark-required::after { content: \"*\"; color: red !important; }","title":"Introspection - DSL: Smokestack.Dsl","ref":"dsl-smokestack.html#introspection"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}